""" Some function I wish some xml library had """

from IPython.display import HTML
from lxml import etree
from bs4 import BeautifulSoup


def render(xml):
    """ Similar to kora.json.render(s) 
    
    Accept both xml string or xml filename
    """
    if xml.endswith('.xml'): # a filename
        with open(xml) as f:
            xml = f.read()

    template = """
    <script src="https://code.jquery.com/jquery-3.3.1.min.js"></script>
    <script src="https://cdn.jsdelivr.net/gh/borsuksoftware/simpleXML/js/simpleXML.js"></script>
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/gh/borsuksoftware/simpleXML/css/simpleXML.css">
    <script>
    $("#output-body").simpleXML({ 
        xmlString: "%s" 
    });
    </script>"""
    return HTML(template % xml
                          .replace('>\n','>')
                          .replace('"','\\"')
               )


def iterparse(source, tag=None, **kw):
    """ memory-efficient parsing for a big xml file
    
    Just etree.iterparse, with my favorite defaults 
    """
    for _, elem in etree.iterparse(source, ("end",), tag=tag, **kw):
        yield elem
        elem.clear()  # auto clear, so, must copy first


def soup(elem):
    """ Convert _Element to BeautifulSoup """
    s = etree.tostring(elem)
    return BeautifulSoup(s, 'lxml')


import os

os.system('pip install pyvis jsonpickle')
import pyvis
from pyvis.network import *
from typing import Union

class Network(pyvis.network.Network):
    """ Same as pyvis but change for Colab notebook """

    def __init__(self, height: Union[int, str] = 500, width: Union[int, str] = 500, **kwargs):
        """ Allow integer as h, w and set notebook=True """
        if type(height)==int:
            height = '%dpx' % height
        if type(width)==int:
            width = '%dpx' % width
        kwargs['height'] = height
        kwargs['width']  = width
        kwargs['notebook'] = True
        super().__init__(**kwargs)

    def _repr_html_(self):
        nodes, edges, height, width, options = self.get_network_data()
        html = self.template.render(height=height, width=width, 
                                    nodes=nodes, edges=edges, 
                                    options=options)
        return html


import os
from time import sleep

from kora import get_ver

# Run once only. If call again, will not restart
if get_ver('tornado') == '4.5.3':
    os.system("pip install -U tornado")  # to 6.0.4

    print("Runtime is now restarting...")
    print("You can ignore the error message [Your session crashed for an unknown reason.]")
    sleep(0.5)
    os._exit(0)  # restart


import os
from IPython.display import HTML


def show_pdb(filename):
    """ render pdb file in 3D """
    from kora.output import nbx
    os.system(f"cp {filename} {nbx}")
    fname = os.path.basename(filename)

    return HTML("""
        <div id=viewer></div>
        <script src='https://cdn.jsdelivr.net/npm/bio-pv/bio-pv.min.js'></script>
        <script>
        var viewer = pv.Viewer(document.getElementById('viewer'));
        function load_pdb(url) {
            pv.io.fetchPdb(url, function(structure) {
                viewer.cartoon('protein', structure, { color : color.ssSuccession() });
                viewer.centerOn(structure);
            });
        }
        load_pdb("/nbextensions/%s")
""" % fname)



import os
from IPython.core.magic import register_line_magic

os.system('wget -qO tldr https://github.com/dbrgn/tealdeer/releases/download/v1.3.0/tldr-linux-x86_64-musl')
os.system('chmod +x tldr')
os.system('mv tldr /usr/local/bin')
os.system('tldr --update')  # need once

@register_line_magic
def tldr(line):
    get_ipython().system('tldr '+line)



""" What I want to add to pandas 

Initially just ability to read and write to google sheet
"""

import re
from pandas import *
import gspread_dataframe


def auth_gsheet():
    from google.colab import auth
    auth.authenticate_user()
    import gspread
    from oauth2client.client import GoogleCredentials as GC
    gsheet = gspread.authorize(GC.get_application_default())
    return gsheet


def gsheet_open(source):
    """ Open existing title/id/url, or create a new one
    
    Return the first sheet (sheet1)
    """
    g = auth_gsheet()
    # a URL ?
    if source.startswith('http'):
        return g.open_by_url(source).sheet1
    # a file_id ?
    if re.match(r'1[-\w]{43}$', source):
        return g.open_by_key(source).sheet1
    # otherwise, it's a title
    try:
        return g.open(source).sheet1
    except:
        return g.create(source).sheet1


def read_sheet(source, **kw):
    """ Imitate pd.read_csv()
    
    Source can be title, fild_id, or URL
    """
    sheet = gsheet_open(source)
    return (gspread_dataframe
            .get_as_dataframe(sheet, **kw)
            .dropna(0, 'all').dropna(1, 'all')
            .fillna(''))
    

def _df_to_sheet(self, title, 
                 header=True, index=False, resize=True, 
                 public=False):
    sheet = gsheet_open(title)
    (gspread_dataframe
     .set_with_dataframe(sheet, self,
                        include_index=index,
                        include_column_header=header,
                        resize=resize)
    )
    if public:
        sheet.spreadsheet.share(None, 'anyone', 'reader')
    return 'https://docs.google.com/spreadsheets/d/' + sheet.spreadsheet.id

DataFrame.to_sheet = _df_to_sheet



import os
from IPython.core.magic import register_line_magic

os.system('wget -qO bat.deb https://github.com/sharkdp/bat/releases/download/v0.15.4/bat_0.15.4_amd64.deb')
os.system('dpkg -i bat.deb')
os.remove('bat.deb')

# set default display in the config file
os.makedirs('/root/.config/bat')
with open('/root/.config/bat/config', "w") as f:
    f.write('''
--theme="ansi-light"
--wrap=never
''')

@register_line_magic
def bat(line):
    get_ipython().system('bat '+line)



"""
ORCHID dataset

For Thai POS tagging
"""
from urllib.request import urlretrieve
from kora.bs import Soup


def download():
    base = 'https://raw.githubusercontent.com/korakot/thainlp/master/'
    filenames = ['orchid_pos.txt', 'xmlchid.xml']
    for fname in filenames:
        url = base + fname 
        urlretrieve(url, fname)
    print('orchid_pos.txt : Pairs of word & pos')
    print('xmlchid.xml    : ORCHID converted to XML (by K. Vee)')
    return filenames

def list_docs():
    return Soup('xmlchid.xml').select('document')

def list_sents():
    return Soup('xmlchid.xml').select('sentence')

def list_words():
    return Soup('xmlchid.xml').select('word')



import os

os.system('wget -qO mysql.deb  https://dev.mysql.com/get/mysql-apt-config_0.8.15-1_all.deb')
os.system('dpkg -i mysql.deb')
os.remove('mysql.deb')
os.system('apt update')
os.system('apt install mysql-server')
get_ipython().system_raw('mysqld_safe &')

os.system('pip install -U ipython-sql pymysql -q')



from urllib.request import urlretrieve
from matplotlib import *  # font_manager, ft2font, rc


def add_thai_font():
    url = 'https://raw.githubusercontent.com/korakot/kora/40a2bb269383501965c0745055f1ad4f23e167a3/THSarabunChula.ttf'
    name = add_font(url)
    set_font(family=name, size=18)

add_font_thai = add_thai_font

def add_font(url: str) -> str:
    if url.startswith('http'):
        filename = urlretrieve(url)[0]  # in /tmp/...
    else:
        filename = url   # already manual download to colab
    name = get_font_name(filename)
    font_manager.fontManager.addfont(filename)
    return name


def get_font_name(filename: str):
    font = ft2font.FT2Font(filename)
    return font.family_name


def set_font(**kw):
    rc('font', **kw) # family, size


# set animation suitable for Colab
rc('animation', html='jshtml')


def use_svg(enable=True):
    """ Set figure_format to SVG """
    format = 'svg' if enable else 'png'
    magic = get_ipython().run_line_magic
    magic('config', f"InlineBackend.figure_format = '{format}'")


import os

# install PostgreSQL 10
os.system("apt install postgresql postgresql-contrib")
os.system("service postgresql start")
os.system("sudo -u postgres psql -c 'CREATE USER root WITH SUPERUSER'")

# update %%sql and add pg special commands
os.system('pip install -U ipython-sql')
os.system('pip install pgspecial')
os.system('pip install psycopg2-binary')  # avoid warning

# config for %%sql
magic = get_ipython().run_line_magic
magic('load_ext', 'sql')
magic('config', 'SqlMagic.displaycon=False')
magic('config', 'SqlMagic.feedback=False')
magic('config', 'SqlMagic.autopandas=True')
magic('sql', 'postgresql+psycopg2://@/postgres')



""" Utility functions not available in google.colab.drive
    Mainly use pydrive
"""
from os.path import basename
from urllib.request import urlretrieve


def auth_drive():
    from pydrive.auth import GoogleAuth
    from pydrive.drive import GoogleDrive
    from oauth2client.client import GoogleCredentials as GC
    from google.colab import auth
    auth.authenticate_user()
    gauth = GoogleAuth()
    gauth.credentials = GC.get_application_default()
    return GoogleDrive(gauth)


def get_file(filename):
    """ Get this file if exists
    Or create a new one. Does not support folder, at root only.
    """
    drive = auth_drive()
    q = f"title = '{filename}' and 'root' in parents"
    res = drive.ListFile({'q': q}).GetList()
    if res:
        fid = res[0]['id']
        return drive.CreateFile({'id': fid})
    else:
        return drive.CreateFile({'title': filename})


def upload_public(filename):
    f = get_file(basename(filename))
    f.SetContentFile(filename)
    f.Upload()
    f.InsertPermission({'type': 'anyone', 'value': 'anyone', 'role': 'reader'})
    return 'https://drive.google.com/uc?id=' + f.get('id')  # direct link


def download_folder(folder_id):
    """ I copy this one from my old code, use drive API directly (not pydrive) """
    # authenticate
    from google.colab import auth
    auth.authenticate_user()
    # get folder_name
    from googleapiclient.discovery import build
    service = build('drive', 'v3')
    folder_name = service.files().get(fileId=folder_id).execute()['name']
    # install library 
    url = 'https://github.com/segnolin/google-drive-folder-downloader/raw/master/download.py'
    path = '/usr/local/lib/python3.6/dist-packages/download.py'
    urlretrieve(url, path)
    # recursive download
    import download
    download.download_folder(service, folder_id, './', folder_name)
    return folder_name


"""
Vega Wordcloud, with tooltips
"""
import os
from collections import Counter
from math import sqrt

os.system("pip install -U pyyaml")
from altair.vega import Vega
Vega.renderers.enable('colab')


mark_spec = {
    "type": "text",
    "from": {"data": "table"},
    "encode": {
        "enter": {
            "text": {"field": "word"},
            "fill": {"scale": "color", "field": "word"},
            "tooltip": {"field": "text"}
        },
    },
    "transform": [{
        "type": "wordcloud",
        "size": [500, 300],  # to fill
        "text": {"field": "text"},
        "rotate": {"field": "datum.angle"},
        # "font": "Sarabun New"
        "fontSize": {"field": "datum.size"},
    }]
}


spec = {
    "name": "wordcloud",
    "width": 500,
    "height": 300,
    "data" : [{
        'name'  : 'table',
        'values': [], # to fill
        'transform': [{
            "type": "formula", "as": "angle",
            "expr": "0"
        }]
    }],
    "scales": [{
        "name": "color",
        "type": "ordinal",
        "range": [],  # to fill
    }],
    "marks": [mark_spec],
}

angle2expr = {
    0: '0',
    45: "[-45, 0, 45][~~(random() * 3)]",
    90: "[0, 90][~~(random() * 2)]",
}


def plot(words, height=300, width=500, 
              size=10, fmin=1, angle=0,
              colors=["#d5a928", "#652c90", "#939597"]):
    """
    size: smallest font to show
    fmin: smallest frequency to show
    angle: 0, 45, or 90
    colors: will alternate
    """
    cnt = Counter(words)   # words can also be a Counter() or dict()
    values = []
    for k, v in cnt.items():
        if v < fmin: continue
        values.append({
            'word': k,
            'text': f"{k} - {v}",    # จะเปลี่ยน label ก็ได้
            'size': size*sqrt(v),
        })
    spec['data'][0]['values'] = values
    spec['data'][0]['transform'][0]['expr'] = angle2expr.get(angle, '0')
    # other parameters
    spec['width'] = width
    spec['height'] = height
    spec['marks'][0]['transform'][0]['size'] = [width, height]
    spec['scales'][0]['range'] = colors
    return Vega(spec)



import os
os.system("pip install trimesh")
from trimesh import Trimesh, PointCloud, Scene, voxel, unitize, tol, load, \
                    load_mesh, load_path, load_remote, primitives, \
                    transform_points, available_formats


# auto display itself, no need to call show()
Trimesh._repr_html_ = lambda self: self.show().data